/* $Workfile: OtxHdlc.h $   $Revision: 23 $    $Modtime: 7/17/00 5:00p $  */
#ifndef OTX_HDLC_H
#define OTX_HDLC_H
/*******************************************************************************

  @doc PublicHwApi OtxHdlc

  Copyright (C) 1998, 1999 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxHdlc | Prototypes for the Logical HDLC Device (Sender/Receiver) Functions

  @contents2
  OtxHdlc.h |

  @index func | OtxHdlc

  @contents2
  OTX Logical Hdlc Sender/Receiver Functions - OtxHdlcXXX() | The API functions for the Logical High-level Data Link Controller (HDLC) device

  @head3 OtxHdlcXXX() Function Reference |

  @index func | OtxHdlc

*******************************************************************************/

#ifdef __cplusplus
  extern "C" {
#endif

#include "OtxType.h"
#include "OtxDef.h"

#include "OtxHdlcD.h"

/*******************************************************************************

  @func
    Initializate the HDLC Sender.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>

  @xref
    <f OtxHdlcReceiverInit>
    <nl><t OtxHdlcFrameFillTypeE>
    <nl><t OtxHdlcCrcModeE>
    <nl><t OtxHdlcTxRateE>

*******************************************************************************/
OTX_RESULT OtxHdlcSenderInit(
  IN OTX_HANDLE  hHdlcDevice,                 // @parm Handle to logical HDLC Filter Device
  IN OtxHdlcFrameFillTypeE eFrameFillType,    // @parm Fill character(s) to be send between HDLC frames. See also <t OtxHdlcFrameFillTypeE>
  IN OTX_INT32   nInterFrameTimeFill,         // @parm Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
  IN OtxHdlcCrcModeE eCrcMode,                // @parm Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none. See also <t OtxHdlcCrcModeE>
  IN OTX_BOOL    bBitInversion,               // @parm If OTX_TRUE then all bits are inverted when transmitted and before received
  IN OTX_BOOL    bAppendTxCrc,                // @parm If OTX_TRUE the HDLC device will append the CRC to the frame to be transmitted. If OTX_FALSE, the CRC is generated with software
  IN OtxHdlcTxRateE eTxRate                   // @parm HDLC Transmission Rate for V.110 and X.30 modes. See also <t OtxHdlcTxRateE>
);



/*******************************************************************************

  @func
    Initializate the HDLC Receiver.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>

  @xref
    <f OtxHdlcSenderInit>
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_SENDER>
    <nl><t OtxHdlcFrameFillTypeE>
    <nl><t OtxHdlcCrcModeE>
    <nl><t OtxHdlcTxRateE>

*******************************************************************************/
OTX_RESULT OtxHdlcReceiverInit(
  IN OTX_HANDLE  hHdlcDevice,                 // @parm Handle to logical HDLC Filter Device
  IN OtxHdlcFrameFillTypeE eFrameFillType,    // @parm Fill character(s) to be send between HDLC frames. See also <t OtxHdlcFrameFillTypeE>
  IN OTX_INT32   nInterFrameTimeFill,         // @parm Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
  IN OtxHdlcCrcModeE eCrcMode,                // @parm Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none. See also <t OtxHdlcCrcModeE>
  IN OTX_BOOL    bBitInversion,               // @parm If OTX_TRUE then all bits are inverted when transmitted and before received
  IN OTX_BOOL    bExcludeRxCrc,               // @parm If OTX_TRUE the HDLC device will remove the CRC from the received frame, if OTX_FALSE the CRC is passed on to the receiving software
  IN OtxHdlcTxRateE eTxRate,                  // @parm HDLC Transmission Rate for V.110 and X.30 modes. See also <t OtxHdlcTxRateE>
  IN OTX_BOOL    bEventForEveryFrame          // @parm If OTX_TRUE the HDLC device will generate an event for every received frame, if OTX_FALSE then the HDLC device will only generate an event when one or more frames have been received since the last call to <f OtxHdlcReadData> or <f OtxHdlcReadDataEx>.
);



/*******************************************************************************

  @func
    Sends transparent data or HDLC frames on a configured HDLC pipe.
  @xref
    <f OtxHdlcSS7SendData>

*******************************************************************************/
OTX_RESULT OtxHdlcSendData(
  IN OTX_HANDLE  hHdlcDevice, // @parm Handle to the HDLC Sender Device
  IN OTX_UINT8   *sqData,     // @parm Data to be sent
  IN OTX_UINT32  nDataLen,    // @parm Length of the data to be send
  IN OTX_TASK_REF nTaskRef,   // @parm See <t OTX_TASK_REF>
  IN OTX_TIME nWaitMaxMs      // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Flushes (empties) the queue of HDLC frames waiting to be transmitted (if any).
  @xref
    <f OtxHdlcSendData>

*******************************************************************************/
OTX_RESULT OtxHdlcSenderFlush(
  IN OTX_HANDLE  hHdlcDevice  // @parm Handle to the HDLC Sender Device
);



/*******************************************************************************

  @func
    Read data received by the HDLC receiver

  @rdesc Returns one of the following values:

    @flag OTX_S_OK | Hdlc Data was fetched
    @flag OTX_S_FALSE | No Hdlc Data available at this moment
    @flag OTX_E_BUFFER_TO_SMALL | The supplied user data buffer is too small to fit the received HDLC frame

  @xref
    <f OtxHdlcReceiverInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_RECEIVER>
    <nl><f OtxHdlcSendData>
    <nl><f OtxHdlcReadDataEx>

*******************************************************************************/
OTX_RESULT OtxHdlcReadData(
  IN  OTX_HANDLE hHdlcDevice,    // @parm Handle to the HDLC Receiver Device
  OUT OTX_UINT8  *pDataBuf,      // @parm Data buffer for the received data
  IN  OTX_UINT32 nBufLen,        // @parm Length of the user data buffer
  OUT OTX_UINT32 *pnFrameLen,    // @parm Length of the received HDLC frame
  OUT OTX_UINT32 *pnFrameSeq,    // @parm Sequence number of the received HDLC frame (matches the number in the event queue)
  OUT OtxHdlcFrameEndCodeE *pnFrameEndCode, // @parm Frame Status Code
  OUT OTX_DATETIME *pnTimeStamp  // @parm Timestamp of the last received octet of the frame
);


/*******************************************************************************

  @func
    Read multiple data frames received by the HDLC receiver.

  @rdesc Returns one of the following values:

    @flag OTX_S_OK | Hdlc Data was fetched
    @flag OTX_S_FALSE | No Hdlc Data available at this moment
    @flag OTX_E_BUFFER_TO_SMALL | The supplied user data buffer is too small to fit the first HDLC frame info and data 

  @comm
    The data in the returned buffer is organized so that it always starts with a frame info struct, 
    OtxHdlcFrameInfoS, after which we have the HDLC data. The next OtxHdlcFrameInfoS follows shortly
    after the last data byte. In order to find the next frame, all frames info structs (except the last
    one) contain a field with the of the relative index of the start of the next frame. The index is
    a byte index relative to the start of the current frame's info struct. I.e. if we have a pointer,
    p1, to the beginning of the first info struct in the buffer we get a pointer to the second struct, 
    p2, by adding the byte offset found in the current info struct, i.e.:

    OtxHdlcFrameInfoS p1 = buffer;
    OtxHdlcFrameInfoS p2 = OTX_STRUCT_ADD_BYTE_OFFSET(OtxHdlcFrameInfoS, p1, p1->nOffsetNext);

    This dispenses with the need to worry about the byte alignments of the OtxHdlcFrameInfoS structs.

    The last frame's nOffsetNext field will be zero

  @xref
    <f OtxHdlcReceiverInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_RECEIVER>
    <nl><f OtxHdlcSendData>
    <nl><f OtxHdlcReadData>
    <nl><t OtxHdlcFrameInfoS>

*******************************************************************************/
OTX_RESULT OtxHdlcReadDataEx(             
  IN  OTX_HANDLE  hHdlcDevice,            // @parm Handle to the HDLC Receiver Device
  OUT OTX_UINT8   *psqInfoAndDataBuffer,  // @parm Data buffer for the received data and info for all received frames
  IN  OTX_UINT32  cbBufferCapacity,       // @parm Length of the buffer
  IN  OTX_UINT32  coMaxFrames,            // @parm Maximum number of HDLC frames to be read by this call. A count of zero means read until buffer full
  IN  OTX_BOOL    bTruncate,              // @parm Utilize whole buffer even if it means that the last frame is truncated and the remainder discarded
  OUT OTX_UINT32  *pcoFrames              // @parm Number of frames actually placed in buffer upon return
);


/*******************************************************************************

  @func
    Get the status and statistics of the incoming SS7 link on the
    specified pipe

  @xref
    <f OtxHdlcSS7SetFilter>
    <nl><f OtxHdlcSS7GetFilter>
    <nl><t OtxHdlcSS7FisuS>

*******************************************************************************/
OTX_RESULT OtxHdlcSS7GetReceiveStatus(
  IN  OTX_HANDLE  hHdlcDevice,    // @parm Handle to the HDLC Device
  OUT OTX_UINT32  *pnFisuCount,   // @parm Number of FISUs received
  OUT OTX_UINT32  *pnLssuCount,   // @parm Number of LSSUs received
  OUT OTX_UINT32  *pnMsuCount,    // @parm Number of MSUs received
  OUT OtxHdlcSS7FisuS *pFisu      // @parm The last received FISU on the pipe. See also <t OtxHdlcSS7FisuS>
);



/*******************************************************************************

  @func
    Set the filter mask for the filtering out FISUs, LSSUs, and/or MSUs
  @xref
    <f OtxHdlcSS7GetFilter>

*******************************************************************************/
OTX_RESULT OtxHdlcSS7SetFilter(
  IN OTX_HANDLE  hHdlcDevice,  // @parm Handle to the HDLC Pipe
  IN OTX_UINT32  filterMask    // @parm Filter Mask (see HDLC_SS7_FILTER_XXXX Macros)
);


/*******************************************************************************

  @func
    Returns the currently active filter mask for filtering
    out FISUs, LSSUs, and/or MSUs
  @xref
    <f OtxHdlcSS7SetFilter>

*******************************************************************************/
OTX_RESULT OtxHdlcSS7GetFilter(
  IN  OTX_HANDLE  hHdlcDevice,  // @parm Handle to the HDLC Device
  OUT OTX_UINT32 *pFilterMask   // @parm Currently active Filter Mask (see HDLC_SS7_FILTER_XXXX Macros)
);



/*******************************************************************************

  @func
    Send a message (LSSU, MSU, or arbitrary data) on a SS7 pipe. After
    the sending of the message has been completed, continues sending
    the new FISU provided as a parameter to the function.
  @xref
    <f OtxHdlcSendData>

*******************************************************************************/
OTX_RESULT OtxHdlcSS7SendData(
  IN OTX_HANDLE  hHdlcDevice, // @parm Handle to the Logical HDLC Device
  IN OTX_UINT8 *psqData,      // @parm Data (MSU, LSSU, or other arbitrary frame) to be sent
  IN OTX_UINT32  nDataLen,    // @parm Length of the data to be sent
  OtxHdlcSS7FisuS *pFisu,     // @parm Next FISU to be sent after the data
  IN OTX_TASK_REF nTaskRef,   // @parm See <t OTX_TASK_REF>
  IN OTX_TIME nWaitMaxMs      // @parm See <t OTX_TIME>
);



/*******************************************************************************

  @func
    Load an HDLC frame to be repeated for the HDLC pipe and begin transmission. The 
    Repeat Data is an HDLC frame which will be sent continuosly when there is no
    other user data frames to be sent. User data can be sent with <f OtxHdlcSendData> function
    on which case the user data frame in interleaved between two consecutive Repeat Data
    frames. After the sending of user data frame has been completed,
    the HDLC controller will resume the sending of the Repeat Data frame.

    This function can be used to continously send SS7 FISUs or LSSUs.
  @xref
   <f OtxHdlcStopRepeatData>

*******************************************************************************/
OTX_RESULT OtxHdlcStartRepeatData(
  IN OTX_HANDLE  hHdlcDevice, // @parm Handle to the HDLC Device
  IN OTX_UINT8   *psqData,    // @parm Repeat Data frame to be sent
  IN OTX_UINT32  nDataLen,    // @parm Length of the data frame to be sent
  IN OTX_TASK_REF nTaskRef,   // @parm See <t OTX_TASK_REF>
  IN OTX_TIME nWaitMaxMs      // @parm See <t OTX_TIME>
);



/*******************************************************************************

  @func
    Stop sending of Repeat Data frames on the specified HDLC pipe.
  @xref
   <f OtxHdlcStartRepeatData>

*******************************************************************************/
OTX_RESULT OtxHdlcStopRepeatData(
  IN OTX_HANDLE  hHdlcDevice // @parm Handle to the HDLC Device
);



#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998 - 2000

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif // OTX_HDLC_H
